﻿using CarShop.Data;
using CarShop.Repository;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CarShop.Logic
{
    public class AveragesResult
    {
        public string BrandName { get; set; }
        public double AveragePrice { get; set; }
        public override string ToString()
        {
            return $"BRAND = {BrandName}, AVG = {AveragePrice}";
        }
        public override bool Equals(object obj)
        {
            if (obj is AveragesResult)
            {
                AveragesResult other = obj as AveragesResult;
                return this.BrandName == other.BrandName &&
                    this.AveragePrice == other.AveragePrice;
            }
            else return false;
        }
        public override int GetHashCode()
        {
            return HashCode.Combine(BrandName, AveragePrice);
        }

    }
    public interface ICarLogic
    {
        void AddCar(string brandName, string modelName, int price);
        Car GetOneCar(int id);
        IList<Car> GetAllCars();
        bool ChangeCar(int carId, string brandName, string modelName, int price);
        bool RemoveCar(int id);

        IList<AveragesResult> GetBrandAverages();

    }
    public class CarLogic : ICarLogic
    {
        ICarRepository carRepo;
        IBrandRepository brandRepo;
        public CarLogic(ICarRepository carRepo, IBrandRepository brandRepo)
        {
            this.carRepo = carRepo;
            this.brandRepo = brandRepo;
        }

        public void AddCar(string brandName, string modelName, int price)
        {
            var brand = this.brandRepo.GetAll().SingleOrDefault(x => x.Name == brandName);
            if (brand == null) throw new ArgumentException("INVALID BRAND");
            Car newCar = new Car();
            newCar.Brand = brand;
            newCar.BrandId = brand.Id;
            newCar.BasePrice = price;
            newCar.Model = modelName;
            carRepo.Insert(newCar);
        }

        public Car GetOneCar(int id)
        {
            return carRepo.GetOne(id);
        }

        public IList<Car> GetAllCars()
        {
            return carRepo.GetAll().ToList();
        }

        public bool ChangeCar(int carId, string brandName, string modelName, int price)
        {
            var brand = brandRepo.GetAll().SingleOrDefault(x => x.Name == brandName);
            if (brand == null) return false;
            return carRepo.ChangeCar(carId, brand, modelName, price);
        }

        public bool RemoveCar(int id)
        {
            return carRepo.Remove(id);
        }

        public IList<AveragesResult> GetBrandAverages()
        {
            var q = from car in carRepo.GetAll()
                    group car by new { car.Brand.Id, car.Brand.Name } into grp
                    select new AveragesResult()
                    {
                        BrandName = grp.Key.Name,
                        AveragePrice = grp.Average(car => car.BasePrice) ?? 0
                    };
            return q.ToList();
        }
    }

}
