﻿using CarShop.Data;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CarShop.Repository
{
    public abstract class Repository<T> : IRepository<T> where T : class
    {
        protected DbContext ctx;
        public Repository(DbContext ctx)
        {
            this.ctx = ctx;
        }
        public IQueryable<T> GetAll()
        {
            return ctx.Set<T>(); // "Set" as a noun, not as a verb!!!
        }

        public T GetOne(int id)
        {
            return ctx.Set<T>().Find(id);
        }

        public void Insert(T entity)
        {
            ctx.Set<T>().Add(entity);
            ctx.SaveChanges(); // Unit of Work ???
        }

        public bool Remove(int id)
        {
            var entity = GetOne(id);
            if (entity == null) return false;
            ctx.Set<T>().Remove(entity);
            ctx.SaveChanges(); // Unit of Work ???
            return true;
        }
    }

    public class CarRepository : Repository<Car>, ICarRepository
    {
        public CarRepository(DbContext ctx) : base(ctx) { }

        public bool ChangeCar(int id, Brand brand, string modelName, int price)
        {
            var car = GetOne(id);
            if (car == null) return false;
            car.Brand = brand;
            car.BrandId = brand.Id;
            car.Model = modelName;
            car.BasePrice = price;
            ctx.SaveChanges();
            return true;
        }
    }

    public class BrandRepository : Repository<Brand>, IBrandRepository 
    {
        public BrandRepository(DbContext ctx) : base(ctx) { }
    }

}
