﻿using AutoMapper;
using CarShop.Data;
using CarShop.Logic;
using CarShop.Repository;
using CarShop.Web.Models;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;

namespace CarShop.Web.Controllers
{
    // https://docs.microsoft.com/en-us/aspnet/core/web-api/?view=aspnetcore-5.0#apicontroller-attribute
    // [ApiController]
    // [Route("[controller]")] , No route template in startup.cs
    // : ControllerBase
    public class CarsApiController : Controller
    {
        public class ApiResult // Single-point-of-use?
        {
            public bool OperationResult { get; set; }
        }

        ICarLogic logic;
        IMapper mapper;
        public CarsApiController(ICarLogic logic, IMapper mapper)
        {
            this.logic = logic;
            this.mapper = mapper;
        }

        [ActionName("all")]
        [HttpGet]
        // GET CarsApi/all
        public IEnumerable<Models.Car> GetAll()
        {
            var cars = logic.GetAllCars();
            return mapper.Map<IList<Data.Car>, List<Models.Car>>(cars);
        }

        // GET CarsApi/del/5
        [ActionName("del")]
        [HttpGet]
        public ApiResult DelOneCar(int id) 
        {
            return new ApiResult() { OperationResult=logic.RemoveCar(id) };
        }

        // POST CarsApi/add + car
        [HttpPost]
        [ActionName("add")]
        public ApiResult AddOneCar(Models.Car car)
        {
            bool success = true;
            try
            {
                logic.AddCar(car.BrandName, car.Name, car.Price);
            }
            catch (ArgumentException ex)
            {
                success = false;
            }
            return new ApiResult() { OperationResult = success };
        }

        // POST CarsApi/mod  + car
        [HttpPost]
        [ActionName("mod")]
        public ApiResult ModOneCar(Models.Car car)
        {
            return new ApiResult() { 
                OperationResult = logic.ChangeCar(car.Id, car.BrandName, car.Name, car.Price) 
            };
        }
        // Test with: Selenium, Postman
    }
}
