﻿using GalaSoft.MvvmLight.Messaging;
using System.Text.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;

namespace CarShop.Wpf
{
    class MainLogic
    {
        string url = "http://localhost:5922/CarsApi/";
        HttpClient client = new HttpClient();
        JsonSerializerOptions jsonOptions = new JsonSerializerOptions(JsonSerializerDefaults.Web);

        void SendMessage(bool success)
        {
            string msg = success ? "Operation completed successfully" : "Operation failed";
            Messenger.Default.Send(msg, "CarResult");
        }

        public List<CarVM> ApiGetCars()
        {
            string json = client.GetStringAsync(url + "all").Result;
            var list = JsonSerializer.Deserialize<List<CarVM>>(json, jsonOptions);
            // NewtonSoft: JsonConvert.DeserializeObject<List<CarVM>>(json)

            // DO NOT EXECUTE SendMessage(true);
            // DEADLOCK because the event handler will call back this same method!
            return list;
        }

        public void ApiDelCar(CarVM car)
        {
            bool success = false;
            if (car != null)
            {
                string json = client.GetStringAsync(url + "del/" + car.Id.ToString()).Result;
                JsonDocument doc = JsonDocument.Parse(json);
                // NewtonSoft:
                // JObject obj = JObject.Parse(json); return (bool)obj["OperationResult"];

                success = doc.RootElement.EnumerateObject().First().Value.GetRawText() == "true";
            }
            SendMessage(success);
        }
        bool ApiEditCar(CarVM car, bool isEditing)
        {
            if (car == null) return false;
			string myUrl = isEditing ? url + "mod" : url + "add";
			
            Dictionary<string, string> postData = new Dictionary<string, string>();
			if (isEditing) postData.Add("id", car.Id.ToString());
            postData.Add("brandName", car.BrandName);
            postData.Add("name", car.Name);
            postData.Add("price", car.Price.ToString());
            string json = client.PostAsync(myUrl, new FormUrlEncodedContent(postData)).
                Result.Content.ReadAsStringAsync().Result;

            JsonDocument doc = JsonDocument.Parse(json);
            return doc.RootElement.EnumerateObject().First().Value.GetRawText() == "true";
        }

        // Alternative: extract editor to interface dependency
        public void EditCar(CarVM car, Func<CarVM, bool> editor)
        {
            CarVM clone = new CarVM();
            if (car!=null) clone.CopyFrom(car);
            bool? success = editor?.Invoke(clone);
            if (success == true)
            {
                if (car!=null) success = ApiEditCar(clone, true); 
                else success = ApiEditCar(clone, false);
            }
            SendMessage(success == true);
        }

    }
}
